import requests
import time
import titlecase

from grrif_secrets import API_KEY, API_SECRET, SESSION_KEY

artist = ""
title = ""

def hashRequest(obj, secretKey):
    """
    Lifted from https://github.com/bretsky/lastfm-scrobbler/blob/master/lastpy/__init__.py
    MIT License
    Copyright (c) 2019 Noah Huber-Feely
    """
    import hashlib

    string = ''
    items = list(obj.keys())
    items.sort()
    for i in items:
        string += i
        string += obj[i]
    string += secretKey
    stringToHash = string.encode('utf8')
    requestHash = hashlib.md5(stringToHash).hexdigest()
    return requestHash

def scrobble_track(artist, track, timestamp):
    """
    Crafts and sends a POST request to Last.fm's API to scrobble a track.
    """
    url = "http://ws.audioscrobbler.com/2.0/"

    params = {
        "method": "track.scrobble",
        "api_key": API_KEY,
        "artist": artist,
        "chosenByUser": "0",
        "sk": SESSION_KEY,
        "timestamp": str(timestamp),
        "track": track,
        }

    reqhash = hashRequest(params, API_SECRET)
    params["api_sig"] = reqhash

    response = requests.post(url, params=params)

    if response.status_code == 200:
        print("Track scrobbled successfully!")
    else:
        print("Failed to scrobble track.")

def currently_playing(ctime, stop_event):
    """
    Scrobble while streaming from the console (multi-threading coord)
    """
    while not stop_event.is_set():
        start_scrobbling(ctime)

def start_scrobbling(ctime):
    """
    Function to get the currently playing track (every 60 seconds).
    """
    while True:
        response = requests.get("https://www.grrif.ch/live/covers.json")

        # Check if the request was successful
        if response.status_code == 200:
            ltime = response.json()[3].get("Hours")
            if ctime == 0 or ctime != ltime:
                data = response.json()[3]
                title = titlecase.titlecase(data.get("Title"))
                artist = titlecase.titlecase(data.get("Artist"))
                ctime = ltime
                utctime = int(int(time.time() - 30))
                print(f"Currently playing {title} by {artist}.")
                scrobble_track(artist, title, utctime)
            else:
                pass
        else:
            print("Failed to retrieve data from the URL.")

        time.sleep(60)



### [ GRRIF Tools by Julien 'fetzu' Bono ]
## [ IMPORTS ]
import sys
import argparse
from datetime import date, datetime

## [ CONFIGURATION ]
__version__ = "0.8.1"

## [ Is CLI even cooler with argparse? ]
parser = argparse.ArgumentParser(
    description="A set of tools for Cool Cats™. Allows you to archive GRRIF's play history and scrobble it to last.fm (upcoming)."
)

subparsers = parser.add_subparsers(dest="command")

archive_parser = subparsers.add_parser("archive", help="Archive GRRIF's play history.")
archive_parser.add_argument(
    "destination",
    choices=["print", "db", "txt"],
    help="Specify where to archive the play history (print to stdout, save to SQLite database or to text in YYYY/MM/DD.txt file(s)).",
)
archive_parser.add_argument(
    "from_date",
    nargs="?",
    default="2021-01-01",
    help="Specify the start date for the archive in YYYY-MM-DD format. Defaults to 2021-01-01.",
)
archive_parser.add_argument(
    "to_date",
    nargs="?",
    default=date.today().strftime("%Y-%m-%d"),
    help=f"Specify the start date for the archive in YYYY-MM-DD format. Defaults to today ({date.today().strftime('%Y-%m-%d')}).",
)

stats_parser = subparsers.add_parser(
    "stats", help="Get some stats out of the database."
)
stats_subparsers = stats_parser.add_subparsers(dest="stats_command")
artists_parser = stats_subparsers.add_parser(
    "artists", help="Display stats for artists"
)
artists_parser.add_argument(
    "topofthepop",
    choices=["top10", "top25", "top100"],
    help="Display the top 10, 25 or 100 artists.",
)
artists_parser.add_argument(
    "from_date",
    nargs="?",
    default="2021-01-01",
    help="Specify the start date for the stats in YYYY-MM-DD format. Defaults to 2021-01-01.",
)
artists_parser.add_argument(
    "to_date",
    nargs="?",
    default=date.today().strftime("%Y-%m-%d"),
    help=f"Specify the start date for the stats in YYYY-MM-DD format. Defaults to today ({date.today().strftime('%Y-%m-%d')}).",
)
tracks_parser = stats_subparsers.add_parser("tracks", help="Display stats for tracks")
tracks_parser.add_argument(
    "topofthepop",
    choices=["top10", "top25", "top100"],
    help="Display the top 10, 25 or 100 tracks.",
)
tracks_parser.add_argument(
    "from_date",
    nargs="?",
    default="2021-01-01",
    help="Specify the start date for the stats in YYYY-MM-DD format. Defaults to 2021-01-01.",
)
tracks_parser.add_argument(
    "to_date",
    nargs="?",
    default=date.today().strftime("%Y-%m-%d"),
    help=f"Specify the start date for the stats in YYYY-MM-DD format. Defaults to today ({date.today().strftime('%Y-%m-%d')}).",
)

scrobble_parser = subparsers.add_parser(
    "scrobble",
    help="Scrobble to Last.fm.",
)
scrobble_subparsers = scrobble_parser.add_subparsers(dest="scrobble_command")

settings_parser = scrobble_subparsers.add_parser(
    "settings", help="Set your last.fm scrobbling settings"
)
settings_parser.add_argument(
    "API_KEY",
    type=str,
    help="Your last.fm API Key",
)
settings_parser.add_argument(
    "API_SECRET",
    type=str,
    help="Your last.fm API secret",
)
settings_parser.add_argument(
    "SESSION_KEY",
    type=str,
    help="Your last.fm API session key",
)

livescrobble_parser = scrobble_subparsers.add_parser(
    "start", help="Start scrobbling to last.fm now."
)


stream_parser = subparsers.add_parser(
    "play",
    help="Play GRRIF in your terminal!",
).add_argument(
    "quality",
    choices=["mp3_high", "mp3_low", "aac_high"],
    nargs="?",
    default="mp3_high",
    help="Specify streaming quality (default: mp3_high)",
)

args = parser.parse_args()


## [ MAIN ]
def main():
    print(
        "##########################################\n"
        f"##### [ GRRIF Tools version {__version__} ] ######\n"
        "##########################################\n"
    )

    # Displays argparse's help message if no arguments are given
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    if args.command == "archive" or args.command == "stats":
        # Set the base URL to scrape data from
        BASE_URL = "https://www.grrif.ch/recherche-de-titres/?date={}"

        # Set the date range to scrape data for
        START_DATE = datetime.strptime(args.from_date, "%Y-%m-%d")
        END_DATE = datetime.strptime(args.to_date, "%Y-%m-%d")

    # Archive was passed !
    if args.command == "archive":
        # The "save to SQLite database" option was chosen
        if args.destination == "db":
            # Let the user know what we are attempting
            print(
                f"Attempting to archive plays from {START_DATE.strftime('%Y-%m-%d')} to {END_DATE.strftime('%Y-%m-%d')} to a SQLite database."
            )

            # Import the necessary functions
            from .grrif_archiver import plays_to_db

            # Create/open the database
            plays_to_db(BASE_URL, START_DATE, END_DATE)

        # The "save to text files" option was chosen
        if args.destination == "txt":
            # Let the user know what we are attempting
            print(
                f"Attempting to archive plays from {START_DATE.strftime('%Y-%m-%d')} to {END_DATE.strftime('%Y-%m-%d')} to text files."
            )

            # Import the necessary functions
            from .grrif_archiver import plays_to_txt

            # Create/open the database
            plays_to_txt(BASE_URL, START_DATE, END_DATE)

        # The "output data to stdout" option was chosen
        if args.destination == "print":
            # Let the user know what we are attempting
            print(
                f"Attempting to print plays from {START_DATE.strftime('%Y-%m-%d')} to {END_DATE.strftime('%Y-%m-%d')} to stdout."
            )

            # Import the necessary functions
            from .grrif_archiver import plays_to_stdout

            # Create/open the database
            plays_to_stdout(BASE_URL, START_DATE, END_DATE)

    # Stats was passed !
    if args.command == "stats":
        # Import the necessary functions
        from .grrif_stats import topofthepop

        if args.stats_command == "artists":
            if args.topofthepop == "top10":
                topofthepop("artist", "10", START_DATE, END_DATE)
            if args.topofthepop == "top25":
                topofthepop("artist", "25", START_DATE, END_DATE)
            if args.topofthepop == "top100":
                topofthepop("artist", "100", START_DATE, END_DATE)

        if args.stats_command == "tracks":
            if args.topofthepop == "top10":
                topofthepop("artist, title", "10", START_DATE, END_DATE)
            if args.topofthepop == "top25":
                topofthepop("artist, title", "25", START_DATE, END_DATE)
            if args.topofthepop == "top100":
                topofthepop("title", "100", START_DATE, END_DATE)

    # Scrobble was passed !
    if args.command == "scrobble":

        if args.scrobble_command == "settings":
            if args.API_KEY is not None and args.API_SECRET is not None and args.SESSION_KEY is not None:
                # Write the settings to file
                import os
                current_path = os.path.dirname(os.path.abspath(__file__))
                settings_path = os.path.join(current_path, "grrif_secrets.py")
                settings_content = f"API_KEY = '{args.API_KEY}'\nAPI_SECRET = '{args.API_SECRET}'\nSESSION_KEY = '{args.SESSION_KEY}'\n"
                with open(settings_path, "w") as settings_file:
                    settings_file.write(settings_content)
            else:
                print("Invalid number of arguments passed, API Key, API Secret and Session Key are needed.")

        if args.scrobble_command == "start":
            from .grrif_scrobbler import start_scrobbling
            start_scrobbling("0")

    # Play was passed !
    if args.command == "play":
        from .grrif_player import start_playback
        start_playback(args.quality)
 